@@ -8,6 +8,7 @@
 from __future__ import division
 
 from math import sqrt, pi, sin, cos, atan2
+import warnings
 
 from numpy import mean
 import transformations
@@ -91,7 +92,8 @@ class Station(object):
 
     _detectors = None
 
-    def __init__(self, cluster, station_id, position, angle, detectors=None):
+    def __init__(self, cluster, station_id, position, angle, detectors=None,
+                 station_number=None):
         """Initialize station
 
         :param cluster: cluster this station is a part of
@@ -104,12 +106,17 @@ class Station(object):
             Orientation is either 'UD' or 'LR' meaning an up-down or
             left-right orientation of the long side of the detector
             respectively.
+        :param station_number: optional unique identifier for a station
+            this can be used by the cluster to find a specific station
+            and makes it easier to link to a real station. If not given
+            it will be equal to the station_id.
 
         """
         self.cluster = cluster
         self.station_id = station_id
         self.position = position
         self.angle = angle
+        self.station_number = station_number if station_number else station_id
 
         if detectors is None:
             # detector positions for a standard station
@@ -202,7 +209,8 @@ class BaseCluster(object):
         self._x, self._y = position
         self._alpha = angle
 
-    def _add_station(self, position, angle, detectors=None):
+    def _add_station(self, position, angle, detectors=None,
+                     station_number=None):
         """Add a station to the cluster
 
         :param position: tuple of (x, y) values
@@ -213,37 +221,78 @@ class BaseCluster(object):
             Orientation is either 'UD' or 'LR' meaning an up-down or
             left-right orientation of the long side of the detector
             respectively.
+        :param station_number: optional unique identifier for a station
+            this can later be used to find a specific station and makes
+            it easier to link to a real station. If not given it will be
+            equal to the station_id generated by this function. Either
+            use this for all added stations or for none.
 
         Example::
 
             >>> cluster = BaseCluster()
             >>> cluster._add_station((0, 0), pi / 2, [(-5, 0, 'UD'), (5, 0, 'UD')])
+
         """
         # Need to make _stations an instance variable to be able to
         # pickle it.  An assignment takes care of that.
         if self._stations is None:
             self._stations = []
-        # 1-based (0 is reserved, see e.g. use of headers in groundparticlesim)
-        station_id = len(self._stations) + 1
+
+        station_id = len(self._stations)
         self._stations.append(Station(self, station_id, position, angle,
-                                       detectors))
+                                      detectors, station_number))
 
     @property
     def stations(self):
         return self._stations
 
+    def get_station(self, station_number):
+        """Get a station by its number"""
+
+        for station in self._stations:
+            if station_number == station.station_number:
+                return station
+
     def get_xyalpha_coordinates(self):
+        """Get cluster coordinates (x, y, alpha).
+
+        The coordinates should be interpreted as follows: first, the
+        cluster is rotated over angle alpha, around its original center.
+        Than, the cluster is translated to (x, y).
+
+        """
         return self._x, self._y, self._alpha
 
     def get_rphialpha_coordinates(self):
+        """Get cluster coordinates (r, phi, alpha).
+
+        The coordinates should be interpreted as follows: first, the
+        cluster is rotated over angle alpha, around its original center.
+        Than, the cluster is translated to (r, phi).
+
+        """
         r = sqrt(self._x ** 2 + self._y ** 2)
         phi = atan2(self._y, self._x)
         return r, phi, self._alpha
 
     def set_xyalpha_coordinates(self, x, y, alpha):
+        """Set cluster coordinates (x, y, alpha).
+
+        The coordinates should be interpreted as follows: first, the
+        cluster is rotated over angle alpha, around its original center.
+        Than, the cluster is translated to (x, y).
+
+        """
         self._x, self._y, self._alpha = x, y, alpha
 
     def set_rphialpha_coordinates(self, r, phi, alpha):
+        """Set cluster coordinates (r, phi, alpha).
+
+        The coordinates should be interpreted as follows: first, the
+        cluster is rotated over angle alpha, around its original center.
+        Than, the cluster is translated to (r, phi).
+
+        """
         self._x = r * cos(phi)
         self._y = r * sin(phi)
         self._alpha = alpha
@@ -317,6 +366,7 @@ class SingleTwoDetectorStation(BaseCluster):
 
 
 class SingleDiamondStation(BaseCluster):
+
     """Define a cluster containing a single diamond shaped station
 
     Detectors 1, 3 and 4 are in the usual position for a 4 detector
@@ -338,63 +388,62 @@ class SingleDiamondStation(BaseCluster):
 
 
 class ScienceParkCluster(BaseCluster):
-    try:
-        network = sapphire.api.Network()
-        sp_stations = network.stations(subcluster=500)
-        gps_coordinates = {}
-        for station in sp_stations:
-            coordinates = sapphire.api.Station(station['number']).location()
-            gps_coordinates[station['number']] = (coordinates['latitude'],
-                                                  coordinates['longitude'],
-                                                  coordinates['altitude'])
-    except:
-        # 1 day self-survey (8 april 2011) + 506 (Niels, pos from site on
-        # 2 dec, 2011) + 508/509 (from site on 8 jul 2013)
-        gps_coordinates = {501: (52.355924173294305, 4.951144021644267,
-                                 56.102345941588283),
-                           502: (52.355293344895919, 4.9501047083812697,
-                                 55.954367009922862),
-                           503: (52.356254735127557, 4.9529437445598328,
-                                 51.582641703076661),
-                           504: (52.357178777910278, 4.9543838852175561,
-                                 54.622688433155417),
-                           505: (52.357251580629246, 4.9484007564706891,
-                                 47.730995402671397),
-                           506: (52.3571787512, 4.95198605591,
-                                 43.8700314863),
-                           507: (52.3560055099, 4.95147879159,
-                                 56.7735242238),
-                           508: (52.3563513341, 4.95070840124,
-                                 52.51091104),
-                           509: (52.3545582682, 4.95569730394,
-                                 59.942809986)}
 
-    # 502, 505, 508 are now diamond shapes, rotation has less
-    # meaning, need positions of every detector to GPS
-    station_rotations = {501: 135, 502: -15, 503: 45, 504: 175, 505: 86,
-                         506: 267, 507: 0, 508: -135, 509: 135}
+    """A cluster containing stations from the Science Park subcluster
 
-    def __init__(self, stations=range(501, 507)):
+    :param stations: A list of station numbers to include. Only stations
+        from the Science Park subcluster are supported. By default 507
+        and 509 are excluded.
+
+    """
+
+    def __init__(self, stations=[501, 502, 503, 504, 505, 506, 508]):
         super(ScienceParkCluster, self).__init__()
 
-        reference = self.gps_coordinates[501]
+        try:
+            gps_coordinates = {}
+            for station in stations:
+                coordinates = sapphire.api.Station(station).location()
+                gps_coordinates[station] = (coordinates['latitude'],
+                                            coordinates['longitude'],
+                                            coordinates['altitude'])
+        except:
+            warnings.warn('Could not get values form the server, Using '
+                          'hard-coded values.', UserWarning)
+            # 1 day self-survey (8 april 2011) + 506 (Niels, pos from site on
+            # 2 dec, 2011) + 508/509 (from site on 8 jul 2013)
+            gps_coordinates = {
+                501: (52.355924173294305, 4.951144021644267, 56.1023459415882),
+                502: (52.355293344895919, 4.9501047083812697, 55.954367009922),
+                503: (52.356254735127557, 4.9529437445598328, 51.582641703076),
+                504: (52.357178777910278, 4.9543838852175561, 54.622688433155),
+                505: (52.357251580629246, 4.9484007564706891, 47.730995402671),
+                506: (52.3571787512, 4.95198605591, 43.8700314863),
+                507: (52.3560055099, 4.95147879159, 56.7735242238),
+                508: (52.3563513341, 4.95070840124, 52.51091104),
+                509: (52.3545582682, 4.95569730394, 59.942809986)}
+
+        # 502, 505, 508 are now diamond shapes, rotation has less
+        # meaning, need positions of every detector to GPS
+        station_rotations = {501: 135, 502: -15, 503: 45, 504: 175, 505: 86,
+                             506: 267, 507: 0, 508: -135, 509: 135}
+
+        reference = gps_coordinates[501]
         transformation = \
             transformations.FromWGS84ToENUTransformation(reference)
 
         for station in stations:
             easting, northing, up = \
-                transformation.transform(self.gps_coordinates[station])
-            alpha = self.station_rotations[station] / 180 * pi
+                transformation.transform(gps_coordinates[station])
+            alpha = station_rotations[station] / 180 * pi
 
             if station not in [501, 502, 505, 508]:
                 detectors = [(0, 8.66, 'UD'), (0, 2.89, 'UD'),
                              (-5, 0, 'LR'), (5, 0, 'LR')]
-                self._add_station((easting, northing), alpha, detectors)
             elif station == 501:
                 # Precise position measurement of 501
                 detectors = [(0.37, 8.62, 'UD'), (.07, 2.15, 'UD'),
                              (-5.23, 0, 'LR'), (5.08, 0, 'LR')]
-                self._add_station((easting, northing), alpha, detectors)
             elif station == 502:
                 # 502 is (since 17 October 2011) diamond-shaped,
                 # with detector 2 moved to the side in LR orientation.
@@ -404,7 +453,6 @@ class ScienceParkCluster(BaseCluster):
                 b = a * sqrt(3)
                 detectors = [(0., b, 'UD'), (a * 2, b, 'LR'),
                              (a, 0., 'LR'), (-a, 0., 'LR')]
-                self._add_station((easting, northing), alpha, detectors)
             elif station == 505:
                 # 505 is (since 24 April 2013) square-shaped,
                 # detector 1 is moved to the left and detector 2 next to it.
@@ -412,15 +460,15 @@ class ScienceParkCluster(BaseCluster):
                 a = station_size / 2
                 detectors = [(-a, station_size, 'UD'), (a, station_size, 'UD'),
                              (-a, 0., 'LR'), (a, 0., 'LR')]
-                self._add_station((easting, northing), alpha, detectors)
             elif station == 508:
-                # 508 is diamond-shaped,
-                # with detector 2 moved to the side of detector 1 in UD orientation.
+                # 508 is diamond-shaped, with detector 2 moved to the
+                # side of detector 1 in UD orientation.
                 station_size = 10
                 a = station_size / 2
                 b = a * sqrt(3)
                 detectors = [(0., b, 'UD'), (a * 2, b, 'UD'),
                              (-a, 0., 'LR'), (a, 0., 'LR')]
-                self._add_station((easting, northing), alpha, detectors)
             else:
                 raise RuntimeError("Programming error. Station unknown.")
+
+            self._add_station((easting, northing), alpha, detectors, station)
